from __future__ import unicode_literals

import os
import sys
import posixpath

import django
from django.utils import six
from django.utils.six.moves.urllib.parse import unquote
from django.contrib.staticfiles import storage


class ManifestStaticFilesStorage(storage.ManifestStaticFilesStorage):
    """A variant of django's ManifestStaticFilesStorage that don't
    fail on missing files (eg: missing images in css).
    Needed when 3rd party libraries are incorrect.

    Due to a limit of collectstatic we can't have collectstatic to issue
    the problems all in the same point so a better way is to grep them::

       ./manage.py collectstatic 2>&1|grep failed| sort | uniq

    """

    UNHASHED_FILES = []

    def _url(self, hashed_name_func, name, force=False, hashed_files=None):
        """
        Override ._url to use hashed url in development + accepts fake images/css
        """
        try:
            hashed_url = super(ManifestStaticFilesStorage, self)._url(
                hashed_name_func, name, force=True, hashed_files=hashed_files)
        except:
            hashed_url = unquote(name)
            sys.stderr.write("hashing: '%s': failed!\n" % (name))
            if name not in self.UNHASHED_FILES:
                self.UNHASHED_FILES += [name]
        return hashed_url


class ManifestStaticFilesStorage_dj10(storage.ManifestStaticFilesStorage):
    """A variant of django's ManifestStaticFilesStorage that don't
    fail on missing files (eg: missing images in css).
    Needed when 3rd party libraries are incorrect.

    Due to a limit of collectstatic we can't have dollectstatic to issue
    the problems all in the same point so a better way is to grep them::

       ./manage.py collectstatic 2>&1|grep failed| sort | uniq

    """

    UNHASHED_FILES = []

    def url_converter(self, name, template=None):  # noqa
        """
        Returns the custom URL converter for the given file name.
        """
        if template is None:
            template = self.default_template

        def converter(matchobj):
            """
            Converts the matched URL depending on the parent level (`..`)
            and returns the normalized and hashed URL using the url method
            of the storage.
            """
            matched, url = matchobj.groups()
            # Completely ignore http(s) prefixed URLs,
            # fragments and data-uri URLs
            if url.startswith(('#', 'http:', 'https:', 'data:', '//')):
                return matched
            name_parts = name.split(os.sep)
            # Using posix normpath here to remove duplicates
            url = posixpath.normpath(url)
            url_parts = url.split('/')
            parent_level, sub_level = url.count('..'), url.count('/')
            if url.startswith('/'):
                sub_level -= 1
                url_parts = url_parts[1:]
            if parent_level or not url.startswith('/'):
                start, end = parent_level + 1, parent_level
            else:
                if sub_level:
                    if sub_level == 1:
                        parent_level -= 1
                    start, end = parent_level, 1
                else:
                    start, end = 1, sub_level - 1
            joined_result = '/'.join(name_parts[:-start] + url_parts[end:])
            hashed_url = self.get_url_or_log(unquote(joined_result), matched, name, force=True)
            file_name = hashed_url.split('/')[-1:]
            relative_url = '/'.join(url.split('/')[:-1] + file_name)

            # Return the hashed version to the file

            return template % unquote(relative_url)

        return converter

    def get_url_or_log(self, joined_result, matched, name, force):
        """Return an url if possible. Return the original url
        and log the missing filename to self.UNHASHED_FILES

        :arg joined_result: the relative path of the file we're looking for
        :arg force: passed to self.url
        :arg name: the name of the file where we found tha matched object
        :arg matched: the pattern we're trying to hash
        """
        try:
            hashed_url = self.url(unquote(joined_result), force=True)
        except:
            hashed_url = unquote(joined_result)
            sys.stderr.write("hashing: '%s': '%s' failed!\n" % (name, matched))
            self.UNHASHED_FILES += [(name, matched)]
        return hashed_url


if django.VERSION[:2] < (1, 11):
    ManifestStaticFilesStorage = ManifestStaticFilesStorage_dj10
